# FIXME: fix the cases when immediate symbol resolution is not enough.
# For the present, fall back to the old symbol resolution rules for ld.
%define _disable_ld_now 1

# (tpg) disable interall rpm dependency generator
# define _use_internal_dependency_generator 0

# Disable debug rpms.
%define _enable_debug_packages	%{nil}
%define debug_package		%{nil}

%define name		nvidia-current
%define version		396.54

%define priority	9700

%define pkgname64	NVIDIA-Linux-x86_64-%{version}

%define drivername		%{name}
%define driverpkgname		x11-driver-video-%{drivername}
%define modulename		nvidia
# for description and documentation
%define xorg_extra_modules	%{_libdir}/xorg/extra-modules
%define nvidia_driversdir	%{_libdir}/%{drivername}/xorg
%define nvidia_extensionsdir	%{_libdir}/%{drivername}/xorg
%define nvidia_modulesdir	%{_libdir}/%{drivername}/xorg
%define nvidia_libdir		%{_libdir}/%{drivername}
%define nvidia_libdir32		%{_prefix}/lib/%{drivername}
%define nvidia_bindir		%{nvidia_libdir}/bin

%define xorg_conf_backup	%{_sysconfdir}/X11/xorg.conf-pre-%{drivername}.bak

# The entry in Cards+ this driver should be associated with, if there is
# no entry in ldetect-lst default pcitable:
# cooker ldetect-lst should be up-to-date
%define ldetect_cards_name	%nil

%define pkgname %{pkgname64}

# http://bugs.rosalinux.ru/show_bug.cgi?id=7791
%if %{mdvver} >= 201610
%bcond_without external_glvnd
%else
%bcond_with external_glvnd
%endif

# Other packages should not require any NVIDIA libraries, and this package
# should not be pulled in when libGL.so.1 is required
%if %{_use_internal_dependency_generator}
%define __noautoprov '(.*)\\.so(.*)|devel\\(libGL(.*)|devel\\(libEGL.*'
%define common_requires_exceptions libGLcore\\.so|libnv.*\\.so|libGL(.*)
%else
%define _provides_exceptions \\.so
%define common_requires_exceptions libGLcore\\.so\\|libnv.*\\.so
%endif

# (anssi) Allow installing of 64-bit package if the runtime dependencies
# of 32-bit libraries are not satisfied. If a 32-bit package that requires
# libGL.so.1 is installed, the 32-bit mesa libs are pulled in and that will
# pull the dependencies of 32-bit nvidia libraries in as well.
%if %{_use_internal_dependency_generator}
%define __noautoreq '%{common_requires_exceptions}|lib.*so\\.[^(]+(\\([^)]+\\))?$'
%else
%define __noautoreq %{common_requires_exceptions}\\|lib.*so\\.[^(]\\+\\(([^)]\\+)\\)\\?$
%endif

# (anssi) Workaround wrong linking as of 310.19.
# This will probably be fixed soon (at least similar issues have been in the past).
# https://devtalk.nvidia.com/default/topic/523762/libnvidia-encode-so-310-19-has-dependency-on-missing-library/
%define _exclude_files_from_autoreq libnvidia-encode.so.%version

Summary:	NVIDIA proprietary X.org driver and libraries, current driver series
Name:		%{name}
Version:	%{version}
Release:	1

Source0:	http://download.nvidia.com/XFree86/Linux-x86_64/%{version}/%{pkgname64}.run
# GPLv2 source code; see also http://cgit.freedesktop.org/~aplattner/
Source2:	http://download.nvidia.com/XFree86/nvidia-settings/nvidia-settings-%{version}.tar.bz2
Source3:	http://download.nvidia.com/XFree86/nvidia-xconfig/nvidia-xconfig-%{version}.tar.bz2
Source4:	http://download.nvidia.com/XFree86/nvidia-modprobe/nvidia-modprobe-%{version}.tar.bz2
Source5:	http://download.nvidia.com/XFree86/nvidia-persistenced/nvidia-persistenced-%{version}.tar.bz2

Source100:	%{name}.rpmlintrc
Source101:	README.urpmi

# Starting from v. 352.x, the build system checks if the kernel provides
# phys_to_dma() function and if so, tries to use that function. However,
# phys_to_dma() is nether exported nor inline on x86-32 with
# CONFIG_X86_DMA_REMAP=y (selected by CONFIG_STA2X11), so a non-functional
# driver is built then.
# However, phys_to_dma() does nothing special except for some exotic
# hardware. If the function is missing, the driver seems to do the necessary
# conversions right, so let us assume it is missing.
Patch1:		%{name}-no-phys_to_dma.patch

# Allow building the drivers with a different GCC version than the kernel.
# Might be risky but it is still better than failed builds of the drivers
# resulting in broken graphics at the users' machines after a GCC update.
Patch2:		no-cc-version-check.patch

License:	Freeware
URL:		http://www.nvidia.com/object/unix.html
Group:		System/Kernel and hardware
Provides:	should-restart = system

# Staring from v396.x, 32-bit x86 systems are not supported by the driver.
ExclusiveArch:	x86_64

BuildRequires:	imagemagick
BuildRequires:	pkgconfig(xrender)
BuildRequires:	pkgconfig(gtk+-2.0)
BuildRequires:	pkgconfig(xv)
BuildRequires:	pkgconfig(gl)
BuildRequires:	pkgconfig(egl)
BuildRequires:	pkgconfig(xxf86vm)
BuildRequires:	pkgconfig(vdpau) >= 1.1
BuildRequires:	pkgconfig(libtirpc)
BuildRequires:	pkgconfig(dbus-1)

%description
The proprietary driver for NVidia GPUs, "current" branch.
Note that hybrid graphics ("Optimus") is not supported officially with
nvidia-current.

%package -n %{driverpkgname}
Summary:	NVIDIA proprietary X.org driver and libraries
Group:		System/Kernel and hardware
Requires(post): update-alternatives >= 1.9.0
Requires(postun): update-alternatives >= 1.9.0
Requires:	x11-server-common
Suggests:	%{drivername}-doc-html = %{version}
# Proper support for versioned kmod() was added in 2008.1:
Requires:	kmod(%{modulename}) = %{version}
# At least mplayer dlopens libvdpau.so.1, so the software will not pull in
# the vdpau library package. We ensure its installation here.
Requires:	%{_lib}vdpau1 >= 1.1
# We need grub2 to add nomodeset!
Requires(pre):	grub2 >= 2.00-34

# Just in case some long-lived nvidia package is based on the same version
# of the drivers
Requires:	dkms-%{drivername} = %{version}-%{release}

%if %{with external_glvnd}
Requires:	%{_lib}gldispatch0
Requires:	%{_lib}opengl0
Requires:	%{_lib}glvnd-glx
Requires:	%{_lib}glvnd-egl
Requires:	%{_lib}glvnd-gles
# ldconfig -X in the post script must see these libraries installed:
Requires(post):	%{_lib}glvnd-glx
Requires(post):	%{_lib}glvnd-egl
Requires(post):	%{_lib}glvnd-gles

# Pull the 32-bit GLVND libraries in as well, the 32-bit applications
# running on 64-bit systems (e.g. Steam) might need them.
Requires:	libgldispatch0
Requires:	libopengl0
Requires:	libglvnd-glx
Requires:	libglvnd-egl
Requires:	libglvnd-gles
# ldconfig -X in the post script must see these libraries installed:
Requires(post):	libglvnd-glx
Requires(post):	libglvnd-egl
Requires(post):	libglvnd-gles
%endif

Conflicts:	nvidia-current-cuda-opencl <= 295.59-1
Suggests:	nvidia-additional-settings

%description -n %{driverpkgname}
NVIDIA proprietary X.org graphics driver, related libraries and
configuration tools.

Please use XFdrake to configure your NVIDIA card. The correct packages
will be automatically installed and configured in this case.

%package -n dkms-%{drivername}
Summary:	Kernel module to manage NVIDIA GPUs
Group:		System/Kernel and hardware
Requires:	dkms
Requires(post):	dkms
Requires(preun):	dkms
Requires:	dracut >= 046

# DKMS package for each long-lived branch provides nvidia-driver-long-lived.
# The "conflicts" clause is needed to prevent the situation when several
# drivers from different branches are installed at the same time. Each of
# the drivers will have alias "nvidia" for modprobe, so they cannot
# co-exist.
Conflicts:	nvidia-driver-long-lived

%description -n dkms-%{drivername}
Kernel module to manage NVIDIA GPUs. This
is to be used with the %{driverpkgname} package.

%package -n %{drivername}-devel
Summary:	NVIDIA OpenGL/CUDA development libraries and headers
Group:		Development/C
Requires:	%{driverpkgname} = %{version}-%{release}
Requires:	%{drivername}-cuda-opencl = %{version}-%{release}
Provides:	nvidia-devel = %{version}-%{release}
Requires:	%{_lib}vdpau-devel

%description -n %{drivername}-devel
NVIDIA OpenGL/CUDA headers. This package is not required for normal use.

%package -n %{drivername}-cuda-opencl
Summary:	CUDA and OpenCL libraries for NVIDIA proprietary driver
Group:		System/Kernel and hardware
Provides:	%{drivername}-cuda = %{version}-%{release}
Requires:	kmod(%{modulename}) = %{version}
Conflicts:	x11-driver-video-nvidia-current <= 295.59

%description -n %{drivername}-cuda-opencl
Cuda and OpenCL libraries for NVIDIA proprietary driver. This package is not
required for normal use, it provides libraries to use NVIDIA cards for High
Performance Computing (HPC).

# HTML doc splitted off because of size, for live cds:
%package -n %{drivername}-doc-html
Summary:	NVIDIA html documentation (%{drivername})
Group:		System/Kernel and hardware
Requires:	%{driverpkgname} = %{version}-%{release}

%description -n %{drivername}-doc-html
HTML version of the README.txt file provided in package
%{driverpkgname}.

%prep
%setup -q -c -T -a 2 -a 3 -a 4 -a 5
sh %{SOURCE0} --extract-only

pushd %{pkgname}/
rm -rf usr/src/nv/precompiled
%patch1 -p1
%patch2 -p1
popd

# We need a different dkms.conf because the one supplied in the upstream
# is actually a template. The real file is generated from it by
# nvidia-installer which we do not use.
pushd %{pkgname}/kernel
mv dkms.conf dkms.conf.orig

# Notes from the original dkms.conf:
# ---------------
# By default, DKMS will add KERNELRELEASE to the make command line; however,
# this will cause the kernel module build to infer that it was invoked via
# Kbuild directly instead of DKMS. The dkms(8) manual page recommends quoting
# the 'make' command name to suppress this behavior.
# ---------------
cat > dkms.conf <<EOF
PACKAGE_NAME="nvidia"
PACKAGE_VERSION="%{version}"
BUILT_MODULE_NAME[0]="nvidia"
DEST_MODULE_LOCATION[0]="/kernel/drivers/video"
BUILT_MODULE_NAME[1]="nvidia-modeset"
DEST_MODULE_LOCATION[1]="/kernel/drivers/video"
BUILT_MODULE_NAME[2]="nvidia-drm"
DEST_MODULE_LOCATION[2]="/kernel/drivers/video"
BUILT_MODULE_NAME[3]="nvidia-uvm"
DEST_MODULE_LOCATION[3]="/kernel/drivers/video"
MAKE="'make' KERNEL_UNAME=\${kernelver} IGNORE_XEN_PRESENCE=1 IGNORE_CC_MISMATCH=1 modules"
CLEAN="'make' clean"
AUTOINSTALL="yes"
EOF
popd

rm -f nvidia-settings-%{version}/src/*/*.a

%build
# The code contains dozens on top of dozens on top of dozens of "false" positives
export CFLAGS="%optflags -Wno-error=format-security"
export CXXFLAGS="$CFLAGS"
export LDFLAGS="%{?ldflags}"
%make -C nvidia-settings-%{version}/src/libXNVCtrl
%make -C nvidia-settings-%{version}
%make -C nvidia-xconfig-%{version}
%make -C nvidia-modprobe-%{version}
%make -C nvidia-persistenced-%{version} LIBS="-ltirpc -ldl"

%install

# README.urpmi
cp %{SOURCE101} .

cd %{pkgname}

# dkms
install -d -m755 %{buildroot}%{_usrsrc}/%{drivername}-%{version}-%{release}

# menu entry
install -d -m755 %{buildroot}%{_datadir}/%{drivername}
cat > %{buildroot}%{_datadir}/%{drivername}/rosa-nvidia-settings.desktop <<EOF
[Desktop Entry]
Name=NVIDIA Display Settings
Comment=Configure NVIDIA X driver
Exec=%{_bindir}/nvidia-settings
Icon=%{drivername}-settings
Terminal=false
Type=Application
Categories=GTK;Settings;HardwareSettings;X-MandrivaLinux-System-Configuration;
EOF

install -d -m755	%{buildroot}%{_datadir}/applications
touch			%{buildroot}%{_datadir}/applications/rosa-nvidia-settings.desktop

# icons
install -d -m755 %{buildroot}%{_iconsdir}/hicolor/{16x16,32x32,48x48}/apps
convert -strip nvidia-settings.png -resize 16x16 %{buildroot}%{_iconsdir}/hicolor/16x16/apps/%{drivername}-settings.png
convert -strip nvidia-settings.png -resize 32x32 %{buildroot}%{_iconsdir}/hicolor/32x32/apps/%{drivername}-settings.png
convert -strip nvidia-settings.png -resize 48x48 %{buildroot}%{_iconsdir}/hicolor/48x48/apps/%{drivername}-settings.png

error_fatal() {
	echo "Error: $@." >&2
	exit 1
}

error_unhandled() {
	echo "Warning: $@." >&2
	echo "Warning: $@." >> warns.log
	# cause distro builds to fail in case of unhandled files
	exit 1
}

parseparams() {
	for value in $rest; do
		local param=$1
		[ -n "$param" ] || error_fatal "unhandled parameter $value"
		shift
		eval $param=$value

		[ -n "$value" ] || error_fatal "empty $param"

		# resolve libdir based on $arch
		if [ "$param" = "arch" ]; then
			case "$arch" in
			NATIVE)		nvidia_libdir=%{nvidia_libdir};;
			COMPAT32)	nvidia_libdir=%{nvidia_libdir32};;
			*)		error_fatal "unknown arch $arch"
			esac
		fi
	done
}

add_to_list() {
	# on distro builds, only use .manifest for doc files
	[ "${2#%doc}" = "${2}" ] && return
	local list="$1.files"
	local entry="$2"
	echo $entry >> $list
}

install_symlink() {
	local pkg="$1"
	local dir="$2"
	mkdir -p %{buildroot}$dir
	ln -s $dest %{buildroot}$dir/$file
	add_to_list $pkg $dir/$file
}

install_lib_symlink() {
	local pkg="$1"
	local dir="$2"
	case "$file" in
	libvdpau_*.so)
		# vdpau drivers => not put into -devel
		;;
	*.so)
		pkg=nvidia-devel;;
	esac
	install_symlink $pkg $dir
}

install_file_only() {
	local pkg="$1"
	local dir="$2"
	mkdir -p %{buildroot}$dir
	# replace 0444 with more usual 0644
	[ "$perms" = "0444" ] && perms="0644"
	install -m $perms $file %{buildroot}$dir
}

install_file() {
	local pkg="$1"
	local dir="$2"
	install_file_only $pkg $dir
	add_to_list $pkg $dir/$(basename $file)
}

get_module_dir() {
	local subdir="$1"
	case "$subdir" in
	extensions*)	echo %{nvidia_extensionsdir};;
	drivers/)	echo %{nvidia_driversdir};;
	/)		echo %{nvidia_modulesdir};;
	*)		error_unhandled "unhandled module subdir $subdir"
			echo %{nvidia_modulesdir};;
	esac
}

for file in nvidia.files nvidia-devel.files nvidia-cuda.files nvidia-dkms.files nvidia-html.files; do
	rm -f $file
	touch $file
done

# install files according to .manifest
cat .manifest | tail -n +9 | sort | uniq | while read line; do
	arch=
	style=
	subdir=
	dest=
	nvidia_libdir=
	inherit_path_str=
	glvnd=

	rest=${line%%%%MODULE:*}
	file=${rest%%%% *}
	rest=${rest#* }
	perms=${rest%%%% *}
	rest=${rest#* }
	type=${rest%%%% *}
	[ "${rest#* }" = "$rest" ] && rest= || rest=${rest#* }

	case "$type" in
	CUDA_LIB)
		parseparams arch subdir
		install_file nvidia-cuda $nvidia_libdir/$subdir
		;;
	CUDA_SYMLINK)
		parseparams arch subdir dest
		install_lib_symlink nvidia-cuda $nvidia_libdir/$subdir
		;;
	ENCODEAPI_LIB)
		parseparams arch subdir
		install_file nvidia $nvidia_libdir/$subdir
		;;
	ENCODEAPI_LIB_SYMLINK)
		parseparams arch dest
		install_lib_symlink nvidia $nvidia_libdir
		;;
	EXPLICIT_PATH)
		parseparams dest
		install_file nvidia %{_datadir}/nvidia
		;;
	NVCUVID_LIB)
		parseparams arch subdir
		install_file nvidia-cuda $nvidia_libdir/$subdir
		;;
	NVCUVID_LIB_SYMLINK)
		parseparams arch dest
		install_lib_symlink nvidia-cuda $nvidia_libdir
		;;
	NVIFR_LIB)
		parseparams arch subdir
		install_file nvidia $nvidia_libdir/$subdir
		;;
	NVIFR_LIB_SYMLINK)
		parseparams arch dest
		install_lib_symlink nvidia $nvidia_libdir
		;;
	OPENGL_LIB)
		parseparams arch
		install_file nvidia $nvidia_libdir
		;;
	OPENGL_SYMLINK)
		parseparams arch dest
		install_lib_symlink nvidia $nvidia_libdir
		;;
	GLVND_LIB)
		parseparams arch
%if ! %{with external_glvnd}
		install_file nvidia $nvidia_libdir
%endif
		;;
	GLVND_SYMLINK)
		parseparams arch dest
%if ! %{with external_glvnd}
		install_lib_symlink nvidia $nvidia_libdir
%endif
		;;
	GLX_CLIENT_LIB|EGL_CLIENT_LIB)
		parseparams arch glvnd
%if ! %{with external_glvnd}
		# Skip non-GLVND libraries
		# https://devtalk.nvidia.com/default/topic/915640/unix-graphics-announcements-and-news/multiple-glx-client-libraries-in-the-nvidia-linux-driver-installer-package/
		[ t"${glvnd}" == "tGLVND" ] && install_file nvidia $nvidia_libdir
%endif
		;;
	GLX_CLIENT_SYMLINK|EGL_CLIENT_SYMLINK)
		# Skip non-GLVND libraries
		parseparams arch dest glvnd
%if ! %{with external_glvnd}
		[ t"${glvnd}" == "tGLVND" ] && install_lib_symlink nvidia $nvidia_libdir
%endif
		;;
	TLS_LIB)
		parseparams arch style subdir
		install_file nvidia $nvidia_libdir/$subdir
		;;
	TLS_SYMLINK)
		parseparams arch style subdir dest
		install_lib_symlink nvidia $nvidia_libdir/$subdir
		;;
	UTILITY_LIB)
		# backward-compatibility
		[ -n "${rest}" ] || rest="NATIVE $rest"
		parseparams arch subdir
		install_file nvidia $nvidia_libdir/$subdir
		;;
	UTILITY_LIB_SYMLINK)
		# backward-compatibility
		[ "${rest#* }" != "$rest" ] || rest="NATIVE $rest"
		parseparams arch dest
		install_lib_symlink nvidia $nvidia_libdir
		;;
	OPENCL_LIB)
		[ -n "${rest}" ] || rest="NATIVE $rest"
		parseparams arch subdir
		install_file nvidia $nvidia_libdir/$subdir
		;;
	OPENCL_LIB_SYMLINK)
		[ "${rest#* }" != "$rest" ] || rest="NATIVE $rest"
		parseparams arch subdir dest
		install_lib_symlink nvidia $nvidia_libdir/$subdir
		;;
	OPENCL_WRAPPER_LIB)
		[ -n "${rest}" ] || rest="NATIVE $rest"
		parseparams arch subdir
		install_file nvidia $nvidia_libdir/$subdir
		;;
	OPENCL_WRAPPER_SYMLINK)
		[ "${rest#* }" != "$rest" ] || rest="NATIVE $rest"
		parseparams arch subdir dest
		install_lib_symlink nvidia $nvidia_libdir/$subdir
		;;
	VDPAU_LIB)
		parseparams arch subdir
		case $file in *libvdpau_nvidia.so*);; *) continue; esac
		install_file nvidia $nvidia_libdir/$subdir
		;;
	VDPAU_SYMLINK)
		parseparams arch subdir dest
		case $file in *libvdpau_nvidia.so*);; *) continue; esac
		install_lib_symlink nvidia $nvidia_libdir/$subdir
		;;
	VDPAU_WRAPPER_LIB)
		parseparams arch subdir
		case $file in *libvdpau_nvidia.so*);; *) continue; esac
		install_file nvidia $nvidia_libdir/$subdir
		;;
	VDPAU_WRAPPER_SYMLINK)
		parseparams arch subdir dest
		case $file in *libvdpau_nvidia.so*);; *) continue; esac
		install_lib_symlink nvidia $nvidia_libdir/$subdir
		;;
	XLIB_STATIC_LIB)
		install_file nvidia-devel %{nvidia_libdir}
		;;
	XLIB_SHARED_LIB)
		install_file nvidia %{nvidia_libdir}
		;;
	XLIB_SYMLINK)
		parseparams dest
		install_lib_symlink nvidia %{nvidia_libdir}
		;;
	LIBGL_LA)
		# (Anssi) we don't install .la files
		;;
	XMODULE_SHARED_LIB|GLX_MODULE_SHARED_LIB)
		parseparams subdir
		install_file nvidia $(get_module_dir $subdir)
		;;
	XMODULE_NEWSYM)
		# symlink that is created only if it doesn't already
		# exist (i.e. as part of x11-server)
		case "$file" in
		libwfb.so)
			continue
			;;
		*)
			error_unhandled "unknown XMODULE_NEWSYM type file $file, skipped"
			continue
		esac
		parseparams subdir dest
		install_symlink nvidia $(get_module_dir $subdir)
		;;
	XMODULE_SYMLINK|GLX_MODULE_SYMLINK)
		parseparams subdir dest
		install_symlink nvidia $(get_module_dir $subdir)
		;;
	VDPAU_HEADER)
		continue
		parseparams subdir
		install_file_only nvidia-devel %{_includedir}/%{drivername}/$subdir
		;;
	OPENGL_HEADER|CUDA_HEADER)
		parseparams subdir
		install_file_only nvidia-devel %{_includedir}/%{drivername}/$subdir
		;;
	DOCUMENTATION)
		parseparams subdir
		case $subdir in
		*/html)
			add_to_list nvidia-html "%%doc %{pkgname}/$file"
			continue
			;;
		*/include/*)
			continue
			;;
		esac
		case $file in
		*XF86Config*|*nvidia-settings.png)
			continue;;
		esac
		add_to_list nvidia "%%doc %{pkgname}/$file"
		;;
	MANPAGE|NVIDIA_MODPROBE_MANPAGE)
		parseparams subdir
		case "$file" in
		*nvidia-installer*)
			# not installed
			continue
			;;
		*nvidia-settings*|*nvidia-xconfig*|*nvidia-modprobe*|*nvidia-persistenced*)
			# installed separately below
			continue
			;;
		*nvidia-smi*|*nvidia-cuda-mps-control*)
			# ok
			;;
		*)
			error_unhandled "skipped unknown man page $(basename $file)"
			continue
		esac
		install_file_only nvidia %{_mandir}/$subdir
		;;
	UTILITY_BINARY)
		case "$file" in
		*nvidia-settings|*nvidia-xconfig|*nvidia-persistenced)
			# not installed, we install our own copy
			continue
			;;
		*nvidia-smi|*nvidia-bug-report.sh|*nvidia-debugdump)
			# ok
			;;
		*nvidia-cuda-mps-control|*nvidia-cuda-mps-server)
			# ok
			;;
		*)
			error_unhandled "unknown binary $(basename $file) will be installed to %{nvidia_bindir}/$(basename $file)"
			;;
		esac
		install_file nvidia %{nvidia_bindir}
		;;
	UTILITY_BIN_SYMLINK)
		case $file in nvidia-uninstall) continue;; esac
		parseparams dest
		install_symlink nvidia %{nvidia_bindir}
		;;
	NVIDIA_MODPROBE)
		# A suid-root tool (GPLv2) used as fallback for loading the module and
		# creating the device nodes in case the driver component is running as
		# a non-root user (e.g. a CUDA application). While the module is automatically
		# loaded by udev rules, the device nodes are not automatically created
		# like with regular drivers and therefore this tool is installed on
		# ROSA as well, at least for now.
		;;
	INSTALLER_BINARY)
		# not installed
		;;
	KERNEL_MODULE_SRC)
		# These lines now have an item like "INHERIT_PATH_DEPTH:1"
		# It specifies, how many components of the path to skip.
		# For example, the manifest mat contain the following line:
		#   kernel/nvidia/nv.c 0644 KERNEL_MODULE_SRC INHERIT_PATH_DEPTH:1
		# Skip one component of the path as suggested by INHERIT_PATH_DEPTH:1,
		# and we get "nvidia/nv.c", which should be installed as
		# /usr/src/<driver-version-release>/nvidia/nv.c.
		parseparams inherit_path_str
		if [ $inherit_path_str != "INHERIT_PATH_DEPTH:1" ]; then
			error_unhandled "unexpected option for $file: \"$inherit_path_str\"."
		fi
		dest=$(echo $file | sed 's/[^/]*\///')
		install_file nvidia-dkms %{_usrsrc}/%{drivername}-%{version}-%{release}/$(dirname $dest)
		;;
	DKMS_CONF)
		# We will replace it with our own dkms.conf below but the
		# file must be listed anyway.
		install_file nvidia-dkms %{_usrsrc}/%{drivername}-%{version}-%{release}
		;;
	CUDA_ICD)
		# in theory this should go to the cuda subpackage, but it goes into the main package
		# as this avoids one broken symlink and it is small enough to not cause space issues
		install_file nvidia %{_sysconfdir}/%{drivername}
		;;
	VULKAN_ICD_JSON)
		install_file nvidia %{_sysconfdir}/vulkan/icd.d
		;;
	GLVND_EGL_ICD_JSON)
		install_file nvidia %{_datadir}/glvnd/egl_vendor.d
		;;
	EGL_EXTERNAL_PLATFORM_JSON)
		install_file nvidia %{_datadir}/egl/egl_external_platform.d/
		;;
	APPLICATION_PROFILE)
		parseparams subdir
		# application profile filenames are versioned, we can use a common
		# non-alternativesized directory
		install_file nvidia %{_datadir}/nvidia/$subdir
		;;
	DOT_DESKTOP)
		# we provide our own for now
		;;
	XORG_OUTPUTCLASS_CONFIG)
		# nvidia-drm-outputclass.conf allows to load the appropriate
		# X11 driver automatically if nvidia kernel module is used
		# by the graphics device. Xorg >= 1.16.
		# Not needed in our case, because we create the
		# Xorg config appropriately.
		;;
	*)
		error_unhandled "file $(basename $file) of unknown type $type will be skipped"
	esac
done

[ -z "$warnings" ] || echo "Please file a bug about the above warnings at http://bugs.rosalinux.ru/." >> warns.log

# Create the convenience symlinks. As the packages with nvidia are mutually
# exclusive now, there is no need to use alternatives for this.
ln -s %{nvidia_libdir} %{buildroot}%{_libdir}/nvidia
ln -s %{nvidia_libdir32} %{buildroot}%{_prefix}/lib/nvidia

# confirm SONAME; if something else than libvdpau_nvidia.so or libvdpau_nvidia.so.1, adapt .spec as needed:
[ "$(objdump -p %{buildroot}%{nvidia_libdir}/vdpau/libvdpau_nvidia.so.%{version} | grep SONAME | gawk '{ print $2 }')" = "libvdpau_nvidia.so.1" ]

rm -f %{buildroot}%{nvidia_libdir}/vdpau/libvdpau_nvidia.so.1
rm -f %{buildroot}%{nvidia_libdir32}/vdpau/libvdpau_nvidia.so.1

# vdpau alternative symlink
install -d -m755 %{buildroot}%{_libdir}/vdpau
touch %{buildroot}%{_libdir}/vdpau/libvdpau_nvidia.so.1
install -d -m755 %{buildroot}%{_prefix}/lib/vdpau
touch %{buildroot}%{_prefix}/lib/vdpau/libvdpau_nvidia.so.1

# self-built binaries
install -m755 ../nvidia-settings-%{version}/src/_out/*/nvidia-settings %{buildroot}%{nvidia_bindir}
install -m755 ../nvidia-xconfig-%{version}/_out/*/nvidia-xconfig %{buildroot}%{nvidia_bindir}
install -m755 ../nvidia-persistenced-%{version}/_out/*/nvidia-persistenced %{buildroot}%{nvidia_bindir}
install -m4755 ../nvidia-modprobe-%{version}/_out/*/nvidia-modprobe %{buildroot}%{nvidia_bindir}

# binary alternatives
install -d -m755			%{buildroot}%{_bindir}
touch					%{buildroot}%{_bindir}/nvidia-debugdump
touch					%{buildroot}%{_bindir}/nvidia-settings
touch					%{buildroot}%{_bindir}/nvidia-smi
touch					%{buildroot}%{_bindir}/nvidia-xconfig
touch					%{buildroot}%{_bindir}/nvidia-bug-report.sh
touch					%{buildroot}%{_bindir}/nvidia-modprobe
touch					%{buildroot}%{_bindir}/nvidia-persistenced
touch					%{buildroot}%{_bindir}/nvidia-cuda-mps-control
touch					%{buildroot}%{_bindir}/nvidia-cuda-mps-server
# rpmlint:
chmod 0755				%{buildroot}%{_bindir}/*

# install man pages
install -m644 ../nvidia-settings-%{version}/doc/_out/*/nvidia-settings.1 %{buildroot}%{_mandir}/man1
install -m644 ../nvidia-xconfig-%{version}/_out/*/nvidia-xconfig.1 %{buildroot}%{_mandir}/man1
install -m644 ../nvidia-modprobe-%{version}/_out/*/nvidia-modprobe.1 %{buildroot}%{_mandir}/man1
install -m644 ../nvidia-persistenced-%{version}/_out/*/nvidia-persistenced.1 %{buildroot}%{_mandir}/man1

# bug #41638 - whatis entries of nvidia man pages appear wrong
gunzip %{buildroot}%{_mandir}/man1/*.gz
sed -r -i '/^nvidia\\-[a-z]+ \\- NVIDIA/s,^nvidia\\-,nvidia-,' %{buildroot}%{_mandir}/man1/*.1
cd %{buildroot}%{_mandir}/man1
rename nvidia alt-%{drivername} *
cd -
touch %{buildroot}%{_mandir}/man1/nvidia-xconfig.1%{_extension}
touch %{buildroot}%{_mandir}/man1/nvidia-settings.1%{_extension}
touch %{buildroot}%{_mandir}/man1/nvidia-modprobe.1%{_extension}
touch %{buildroot}%{_mandir}/man1/nvidia-persistenced.1%{_extension}
touch %{buildroot}%{_mandir}/man1/nvidia-smi.1%{_extension}
touch %{buildroot}%{_mandir}/man1/nvidia-cuda-mps-control.1%{_extension}

# cuda nvidia.icd
install -d -m755		%{buildroot}%{_sysconfdir}/OpenCL/vendors
touch				%{buildroot}%{_sysconfdir}/OpenCL/vendors/nvidia.icd

# ld.so.conf
install -d -m755		%{buildroot}%{_sysconfdir}/%{drivername}
%if %{with external_glvnd}
echo "%{_libdir}/libglvnd" >>	%{buildroot}%{_sysconfdir}/%{drivername}/ld.so.conf
%endif
echo "%{nvidia_libdir}" >>	%{buildroot}%{_sysconfdir}/%{drivername}/ld.so.conf

%if %{with external_glvnd}
echo "/usr/lib/libglvnd" >>	%{buildroot}%{_sysconfdir}/%{drivername}/ld.so.conf
%endif
echo "%{nvidia_libdir32}" >>	%{buildroot}%{_sysconfdir}/%{drivername}/ld.so.conf

install -d -m755		%{buildroot}%{_sysconfdir}/ld.so.conf.d
touch				%{buildroot}%{_sysconfdir}/ld.so.conf.d/GL.conf

# modprobe.d
install -d -m755			%{buildroot}%{_sysconfdir}/modprobe.d

# Allow the users to load this kernel module as "nvidia-current" as well.
echo "alias %{drivername} nvidia" > %{buildroot}%{_sysconfdir}/modprobe.d/%{drivername}.conf

# xinit script
install -d -m755 %{buildroot}%{_sysconfdir}/%{drivername}
cat > %{buildroot}%{_sysconfdir}/%{drivername}/nvidia-settings.xinit <<EOF
# to be sourced
#
# Do not modify this file; the changes will be overwritten.
# If you want to disable automatic configuration loading, create
# /etc/sysconfig/nvidia-settings with this line: LOAD_NVIDIA_SETTINGS="no"
#
LOAD_NVIDIA_SETTINGS="yes"
[ -f %{_sysconfdir}/sysconfig/nvidia-settings ] && . %{_sysconfdir}/sysconfig/nvidia-settings
[ "\$LOAD_NVIDIA_SETTINGS" = "yes" ] && %{_bindir}/nvidia-settings --load-config-only
EOF
chmod 0755 %{buildroot}%{_sysconfdir}/%{drivername}/nvidia-settings.xinit
install -d -m755 %{buildroot}%{_sysconfdir}/X11/xinit.d
touch %{buildroot}%{_sysconfdir}/X11/xinit.d/nvidia-settings.xinit

export EXCLUDE_FROM_STRIP="$(find %{buildroot} -type f \! -name nvidia-settings \! -name nvidia-xconfig \! -name nvidia-modprobe \! -name nvidia-persistenced)"

%if %{with external_glvnd}
LIBGL_SUBST=libGLX_nvidia.so.0
%else
LIBGL_SUBST=libGL.so.1
%endif
mv %{buildroot}%{_sysconfdir}/vulkan/icd.d/nvidia_icd.json{.template,}
sed -ri "s,__NV_VK_ICD__,${LIBGL_SUBST},g" %{buildroot}%{_sysconfdir}/vulkan/icd.d/nvidia_icd.json

%post -n %{driverpkgname}

# On the first install, generate xorg.conf, backup the old one, if present.
# We cannot simply add it to the list of files because it needs info from
# the system the driver will run on: BUS ID for the video adapter.
if [ $1 -eq 1 ]; then
	if [ -f %{_sysconfdir}/X11/xorg.conf ]; then
		echo "Warning: %{_sysconfdir}/X11/xorg.conf will be replaced with a new file."
		echo "%{xorg_conf_backup} will be the backup of the original xorg.conf."

		mv %{_sysconfdir}/X11/xorg.conf %{xorg_conf_backup}
	fi

# The first of the video adapters by NVidia.
	DEV_LINE=$(lspci -vnn | grep -E -i '(VGA compatible controller|3D controller|Display controller).*NVIDIA' | head -n 1)
	echo $DEV_LINE | grep -E '^[0-9]+:[0-9]+\.[0-9]+\s+' > /dev/null 2>&1
	if test $? -ne 0; then
		echo "Error: no suitable devices found."
		exit 1
	fi

	BUSID_GPU=$(echo $DEV_LINE | awk '{ split($1, ids, "[.:]"); printf("%d:%d:%d", ids[1], ids[2], ids[3]); }')

	cat > %{_sysconfdir}/X11/xorg.conf <<EOF
Section "ServerFlags"
Option "DontZap" "False"
AllowMouseOpenFail
EndSection

Section "Device"
Identifier "device1"
VendorName "NVIDIA Corporation"
BoardName "Proprietary (nvidia-current)"
Driver "nvidia"
BusID "PCI:$BUSID_GPU"
Option "DPMS"
Option "DynamicTwinView" "false"
Option "AddARGBGLXVisuals"
Option "NoLogo" "1"
Option "Coolbits" "4"
EndSection
EOF
fi

# XFdrake used to generate an nvidia.conf file
[ -L %{_sysconfdir}/modprobe.d/nvidia.conf ] || rm -f %{_sysconfdir}/modprobe.d/nvidia.conf

current_glconf="$(readlink -e %{_sysconfdir}/ld.so.conf.d/GL.conf)"

# owned by libvdpau1, created in case libvdpau1 is installed only just after
# this package
mkdir -p %{_libdir}/vdpau

%{_sbindir}/update-alternatives \
	--install %{_sysconfdir}/ld.so.conf.d/GL.conf gl_conf %{_sysconfdir}/%{drivername}/ld.so.conf %{priority} \
	--slave %{_mandir}/man1/nvidia-settings.1%{_extension} man_nvidiasettings%{_extension} %{_mandir}/man1/alt-%{drivername}-settings.1%{_extension} \
	--slave %{_mandir}/man1/nvidia-xconfig.1%{_extension} man_nvidiaxconfig%{_extension} %{_mandir}/man1/alt-%{drivername}-xconfig.1%{_extension} \
	--slave %{_mandir}/man1/nvidia-smi.1%{_extension} nvidia-smi.1%{_extension} %{_mandir}/man1/alt-%{drivername}-smi.1%{_extension} \
	--slave %{_mandir}/man1/nvidia-cuda-mps-control.1%{_extension} nvidia-cuda-mps-control.1%{_extension} %{_mandir}/man1/alt-%{drivername}-cuda-mps-control.1%{_extension} \
	--slave %{_datadir}/applications/rosa-nvidia-settings.desktop nvidia_desktop %{_datadir}/%{drivername}/rosa-nvidia-settings.desktop \
	--slave %{_bindir}/nvidia-settings nvidia_settings %{nvidia_bindir}/nvidia-settings \
	--slave %{_bindir}/nvidia-smi nvidia_smi %{nvidia_bindir}/nvidia-smi \
	--slave %{_bindir}/nvidia-xconfig nvidia_xconfig %{nvidia_bindir}/nvidia-xconfig \
	--slave %{_bindir}/nvidia-debugdump nvidia-debugdump %{nvidia_bindir}/nvidia-debugdump \
	--slave %{_bindir}/nvidia-bug-report.sh nvidia_bug_report %{nvidia_bindir}/nvidia-bug-report.sh \
	--slave %{_bindir}/nvidia-cuda-mps-control nvidia-cuda-mps-control %{nvidia_bindir}/nvidia-cuda-mps-control \
	--slave %{_bindir}/nvidia-cuda-mps-server nvidia-cuda-mps-server %{nvidia_bindir}/nvidia-cuda-mps-server \
	--slave %{_bindir}/nvidia-modprobe nvidia-modprobe %{nvidia_bindir}/nvidia-modprobe \
	--slave %{_bindir}/nvidia-persistenced nvidia-persistenced %{nvidia_bindir}/nvidia-persistenced \
	--slave %{_sysconfdir}/X11/xinit.d/nvidia-settings.xinit nvidia-settings.xinit %{_sysconfdir}/%{drivername}/nvidia-settings.xinit \
	--slave %{_libdir}/vdpau/libvdpau_nvidia.so.1 %{_lib}vdpau_nvidia.so.1 %{nvidia_libdir}/vdpau/libvdpau_nvidia.so.%{version} \
	--slave %{_sysconfdir}/OpenCL/vendors/nvidia.icd nvidia.icd %{_sysconfdir}/%{drivername}/nvidia.icd \
	--slave %{_prefix}/lib/vdpau/libvdpau_nvidia.so.1 libvdpau_nvidia.so.1 %{nvidia_libdir32}/vdpau/libvdpau_nvidia.so.%{version} \
	--slave %{xorg_extra_modules} xorg_extra_modules %{nvidia_extensionsdir}

# Unlike the drivers from long-lived branches (nvidia<version>)
# nvidia-current is not installed by XFdrake. So, it should perform all
# setup tasks by itself.
%{_sbindir}/update-alternatives --set gl_conf %{_sysconfdir}/%{drivername}/ld.so.conf

# explicit /sbin/ldconfig due to alternatives
/sbin/ldconfig -X

%if "%{ldetect_cards_name}" != ""
[ -x %{_sbindir}/update-ldetect-lst ] && %{_sbindir}/update-ldetect-lst || :
%endif

# Add 'nomodeset' to grub2 string and update grub2.cfg. No need to worry
# about Optimus here: it is not officially supported with nvidia-current.
echo "INFO: Adding 'nomodeset'".
/usr/sbin/grub2-cfg-mod --add nomodeset
/usr/sbin/update-grub2

%postun -n %{driverpkgname}
if [ ! -f %{_sysconfdir}/%{drivername}/ld.so.conf ]; then
  %{_sbindir}/update-alternatives --remove gl_conf %{_sysconfdir}/%{drivername}/ld.so.conf
fi
# explicit /sbin/ldconfig due to alternatives
/sbin/ldconfig -X

if [ $1 -eq 0 ]; then
# Remove the generated xorg.conf.
	rm -f %{_sysconfdir}/X11/xorg.conf
fi

%if "%{ldetect_cards_name}" != ""
[ -x %{_sbindir}/update-ldetect-lst ] && %{_sbindir}/update-ldetect-lst || :
%endif

# Del 'nomodeset' from the boot-time kernel options.
if [ $1 -eq 0 ] ; then
	/usr/sbin/grub2-cfg-mod --del nomodeset
	/usr/sbin/update-grub2
fi

%post -n %{drivername}-cuda-opencl
# Explicit /sbin/ldconfig due to a non-standard library directory.
/sbin/ldconfig -X

%post -n %{drivername}-devel
# Explicit /sbin/ldconfig due to a non-standard library directory.
# The applications like Blender may need to find libcuda.so, for example.
/sbin/ldconfig -X

%postun -n %{drivername}-devel
# Explicit /sbin/ldconfig due to a non-standard library directory.
/sbin/ldconfig -X

%post -n dkms-%{drivername}
/usr/sbin/dkms --rpm_safe_upgrade add -m %{drivername} -v %{version}-%{release}

# Build and install the driver for all available kernels that have devel
# files, no matter how these kernels were installed (from RPMs or manually).
# Do not fail the installation of the package if the build fails for some
# kernels, this may happen during testing, etc.
for kk in /lib/modules/*; do
	kk=$(echo $kk | sed 's/^\/lib\/modules\///');
	if test -d "/lib/modules/$kk/build"; then
# Remove the modules that use the obsolete naming convention first
# to make the upgrade smoother (otherwise they may remain in the system
# and cause trouble).
		rm -f /lib/modules/$kk/kernel/drivers/video/nvidia3*.ko
		rm -f /lib/modules/$kk/kernel/drivers/video/nvidia-current.ko
# Now build and install the newer modules
		/usr/sbin/dkms --rpm_safe_upgrade build -m %{drivername} -v %{version}-%{release} -k $kk &&
		/usr/sbin/dkms --rpm_safe_upgrade install -m %{drivername} -v %{version}-%{release} -k $kk || true
	else
		echo "WARNING: No development files for kernel \"$kk\"" > /dev/stderr;
	fi;
done

# Just in case the user reboots the system too quickly and sync is not
# done automatically.
sync

# rmmod any old driver if present and not in use (e.g. by X)
modprobe -r nvidia > /dev/null 2>&1 || true

%preun -n dkms-%{drivername}
/usr/sbin/dkms --rpm_safe_upgrade remove -m %{drivername} -v %{version}-%{release} --all || true

# rmmod any old driver if present and not in use (e.g. by X)
modprobe -r nvidia > /dev/null 2>&1 || true

%files -n %{driverpkgname} -f %{pkgname}/nvidia.files
# other documentation files are listed in nvidia.files
%doc README.urpmi

# ld.so.conf, modprobe.conf, xinit
%ghost %{_sysconfdir}/ld.so.conf.d/GL.conf
%ghost %{_sysconfdir}/X11/xinit.d/nvidia-settings.xinit
%dir %{_sysconfdir}/%{drivername}
%{_sysconfdir}/modprobe.d/%{drivername}.conf
%{_sysconfdir}/%{drivername}/ld.so.conf
%{_sysconfdir}/%{drivername}/nvidia-settings.xinit
%{_sysconfdir}/%{drivername}/nvidia.icd
%dir %{_sysconfdir}/vulkan/icd.d
%{_sysconfdir}/vulkan/icd.d/nvidia_icd.json
%dir %{_datadir}/glvnd/egl_vendor.d
%{_datadir}/glvnd/egl_vendor.d/10_nvidia.json
%dir %{_datadir}/egl/egl_external_platform.d
%{_datadir}/egl/egl_external_platform.d/10_nvidia_wayland.json
%dir %{_datadir}/nvidia
%{_datadir}/nvidia/nvidia-application-profiles-%{version}-rc
%{_datadir}/nvidia/nvidia-application-profiles-%{version}-key-documentation

%dir %{_sysconfdir}/OpenCL
%dir %{_sysconfdir}/OpenCL/vendors
%ghost %{_sysconfdir}/OpenCL/vendors/nvidia.icd

%ghost %{_bindir}/nvidia-debugdump
%ghost %{_bindir}/nvidia-settings
%ghost %{_bindir}/nvidia-smi
%ghost %{_bindir}/nvidia-xconfig
%ghost %{_bindir}/nvidia-modprobe
%ghost %{_bindir}/nvidia-persistenced
%ghost %{_bindir}/nvidia-bug-report.sh
%ghost %{_bindir}/nvidia-cuda-mps-control
%ghost %{_bindir}/nvidia-cuda-mps-server
%dir %{nvidia_bindir}
%{nvidia_bindir}/nvidia-debugdump
%{nvidia_bindir}/nvidia-settings
%{nvidia_bindir}/nvidia-smi
%{nvidia_bindir}/nvidia-xconfig
%{nvidia_bindir}/nvidia-modprobe
%{nvidia_bindir}/nvidia-persistenced
%{nvidia_bindir}/nvidia-bug-report.sh
%{nvidia_bindir}/nvidia-cuda-mps-control
%{nvidia_bindir}/nvidia-cuda-mps-server

%ghost %{_mandir}/man1/nvidia-xconfig.1%{_extension}
%ghost %{_mandir}/man1/nvidia-settings.1%{_extension}
%ghost %{_mandir}/man1/nvidia-modprobe.1%{_extension}
%ghost %{_mandir}/man1/nvidia-persistenced.1%{_extension}
%ghost %{_mandir}/man1/nvidia-smi.1%{_extension}
%ghost %{_mandir}/man1/nvidia-cuda-mps-control.1%{_extension}
%{_mandir}/man1/alt-%{drivername}-xconfig.1*
%{_mandir}/man1/alt-%{drivername}-settings.1*
%{_mandir}/man1/alt-%{drivername}-modprobe.1*
%{_mandir}/man1/alt-%{drivername}-persistenced.1*
%{_mandir}/man1/alt-%{drivername}-smi.1*
%{_mandir}/man1/alt-%{drivername}-cuda-mps-control.1*

%ghost %{_datadir}/applications/rosa-nvidia-settings.desktop
%dir %{_datadir}/%{drivername}
%{_datadir}/%{drivername}/rosa-nvidia-settings.desktop

%{_iconsdir}/hicolor/16x16/apps/%{drivername}-settings.png
%{_iconsdir}/hicolor/32x32/apps/%{drivername}-settings.png
%{_iconsdir}/hicolor/48x48/apps/%{drivername}-settings.png

%dir %{nvidia_libdir}
%dir %{nvidia_libdir}/tls
%dir %{nvidia_libdir}/vdpau
%{_libdir}/nvidia
%if ! %{with external_glvnd}
%{nvidia_libdir}/libGL.so.1.0.0
%{nvidia_libdir}/libGL.so.1
%{nvidia_libdir}/libGLX.so.0
%{nvidia_libdir}/libGLdispatch.so.0
%{nvidia_libdir}/libOpenGL.so.0
%{nvidia_libdir}/libEGL.so.1
%{nvidia_libdir}/libGLESv1_CM.so.1
%{nvidia_libdir}/libGLESv2.so.2
%endif
%{nvidia_libdir}/libGLX_nvidia.so.%{version}
%{nvidia_libdir}/libGLX_nvidia.so.0
%{nvidia_libdir}/libGLX_indirect.so.0
%{nvidia_libdir}/libnvidia-fatbinaryloader.so.%{version}
%{nvidia_libdir}/libnvidia-ptxjitcompiler.so*
%{nvidia_libdir}/libnvidia-glcore.so.%{version}
%{nvidia_libdir}/libnvidia-cfg.so.%{version}
%{nvidia_libdir}/libnvidia-tls.so.%{version}
%{nvidia_libdir}/libnvidia-glvkspirv.so.%{version}
%{nvidia_libdir}/vdpau/libvdpau_nvidia.so.%{version}
%{nvidia_libdir}/libEGL_nvidia.so.0
%{nvidia_libdir}/libEGL_nvidia.so.%{version}
%{nvidia_libdir}/libGLESv1_CM_nvidia.so.1
%{nvidia_libdir}/libGLESv1_CM_nvidia.so.%{version}
%{nvidia_libdir}/libGLESv2_nvidia.so.2
%{nvidia_libdir}/libGLESv2_nvidia.so.%{version}
%{nvidia_libdir}/libnvidia-eglcore.so.%{version}
%{nvidia_libdir}/libnvidia-glsi.so.%{version}
%{nvidia_libdir}/libnvidia-cfg.so.1
%{nvidia_libdir}/libnvidia-fbc.so.1
%{nvidia_libdir}/libnvidia-fbc.so.%{version}
%{nvidia_libdir}/libnvidia-ifr.so.1
%{nvidia_libdir}/libnvidia-ifr.so.%{version}
%{nvidia_libdir}/libnvidia-ml.so.1
%{nvidia_libdir}/libnvidia-ml.so.%{version}
%{nvidia_libdir}/libvdpau_nvidia.so
%{nvidia_libdir}/tls/libnvidia-tls.so.%{version}
%{nvidia_libdir}/libnvidia-gtk2.so.%{version}
%{nvidia_libdir}/libnvidia-gtk3.so.%{version}
%{nvidia_libdir}/libnvidia-egl-wayland.so.1*
%dir %{nvidia_libdir32}
%dir %{nvidia_libdir32}/tls
%dir %{nvidia_libdir32}/vdpau
%{_prefix}/lib/nvidia
%if ! %{with external_glvnd}
%{nvidia_libdir32}/libGL.so.1.0.0
%{nvidia_libdir32}/libGL.so.1
%{nvidia_libdir32}/libGLX.so.0
%{nvidia_libdir32}/libGLdispatch.so.0
%{nvidia_libdir32}/libOpenGL.so.0
%{nvidia_libdir32}/libEGL.so.1
%{nvidia_libdir32}/libGLESv1_CM.so.1
%{nvidia_libdir32}/libGLESv2.so.2
%endif
%{nvidia_libdir32}/libGLX_nvidia.so.%{version}
%{nvidia_libdir32}/libGLX_nvidia.so.0
%{nvidia_libdir32}/libGLX_indirect.so.0
%{nvidia_libdir32}/libnvidia-fatbinaryloader.so.%{version}
%{nvidia_libdir32}/libnvidia-ptxjitcompiler.so*
%{nvidia_libdir32}/libnvidia-glcore.so.%{version}
%{nvidia_libdir32}/libnvidia-ifr.so.1
%{nvidia_libdir32}/libnvidia-ifr.so.%{version}
%{nvidia_libdir32}/libnvidia-ml.so.1
%{nvidia_libdir32}/libnvidia-ml.so.%{version}
%{nvidia_libdir32}/libnvidia-tls.so.%{version}
%{nvidia_libdir32}/libnvidia-glvkspirv.so.%{version}
%{nvidia_libdir32}/libvdpau_nvidia.so
%{nvidia_libdir32}/vdpau/libvdpau_nvidia.so.%{version}
%{nvidia_libdir32}/tls/libnvidia-tls.so.%{version}
%{nvidia_libdir32}/libnvidia-fbc.so.1
%{nvidia_libdir32}/libnvidia-fbc.so.%{version}
%{nvidia_libdir32}/libEGL_nvidia.so.0
%{nvidia_libdir32}/libEGL_nvidia.so.%{version}
%{nvidia_libdir32}/libGLESv1_CM_nvidia.so.1
%{nvidia_libdir32}/libGLESv1_CM_nvidia.so.%{version}
%{nvidia_libdir32}/libGLESv2_nvidia.so.2
%{nvidia_libdir32}/libGLESv2_nvidia.so.%{version}
%{nvidia_libdir32}/libnvidia-eglcore.so.%{version}
%{nvidia_libdir32}/libnvidia-glsi.so.%{version}

%ghost %{_libdir}/vdpau/libvdpau_nvidia.so.1
# avoid unowned directory
%dir %{_prefix}/lib/vdpau
%ghost %{_prefix}/lib/vdpau/libvdpau_nvidia.so.1

%dir %{nvidia_modulesdir}
%{nvidia_modulesdir}/libnvidia-wfb.so.1
%{nvidia_modulesdir}/libnvidia-wfb.so.%{version}
%{nvidia_extensionsdir}/libglx.so.%{version}
%{nvidia_extensionsdir}/libglx.so
%{nvidia_driversdir}/nvidia_drv.so

%files -n %{drivername}-devel -f %{pkgname}/nvidia-devel.files
%{_includedir}/%{drivername}
%if ! %{with external_glvnd}
%{nvidia_libdir}/libGL.so
%{nvidia_libdir}/libOpenGL.so
%{nvidia_libdir}/libGLX.so
%{nvidia_libdir}/libEGL.so
%{nvidia_libdir}/libGLESv1_CM.so
%{nvidia_libdir}/libGLESv2.so
%endif
%{nvidia_libdir}/libcuda.so
%{nvidia_libdir}/libnvcuvid.so
%{nvidia_libdir}/libnvidia-cfg.so
%{nvidia_libdir}/libnvidia-fbc.so
%{nvidia_libdir}/libnvidia-ifr.so
%{nvidia_libdir}/libOpenCL.so
%{nvidia_libdir}/libnvidia-ml.so
%{nvidia_libdir}/libnvidia-encode.so
%if ! %{with external_glvnd}
%{nvidia_libdir32}/libGL.so
%{nvidia_libdir32}/libOpenGL.so
%{nvidia_libdir32}/libGLX.so
%{nvidia_libdir32}/libEGL.so
%{nvidia_libdir32}/libGLESv1_CM.so
%{nvidia_libdir32}/libGLESv2.so
%endif
%{nvidia_libdir32}/libcuda.so
%{nvidia_libdir32}/libnvcuvid.so
%{nvidia_libdir32}/libnvidia-fbc.so
%{nvidia_libdir32}/libnvidia-ifr.so
%{nvidia_libdir32}/libOpenCL.so
%{nvidia_libdir32}/libnvidia-ml.so
%{nvidia_libdir32}/libnvidia-encode.so


%files -n dkms-%{drivername}
%doc %{pkgname}/LICENSE
%{_usrsrc}/%{drivername}-%{version}-%{release}

%files -n %{drivername}-doc-html -f %{pkgname}/nvidia-html.files

%files -n %{drivername}-cuda-opencl -f %{pkgname}/nvidia-cuda.files
%{nvidia_libdir}/libnvcuvid.so.%{version}
%{nvidia_libdir}/libnvcuvid.so.1
%{nvidia_libdir}/libnvidia-encode.so.1
%{nvidia_libdir}/libnvidia-encode.so.%{version}
%{nvidia_libdir}/libOpenCL.so.1.0.0
%{nvidia_libdir}/libOpenCL.so.1.0
%{nvidia_libdir}/libOpenCL.so.1
%{nvidia_libdir}/libnvidia-compiler.so.%{version}
%{nvidia_libdir}/libcuda.so.%{version}
%{nvidia_libdir}/libcuda.so.1
%{nvidia_libdir}/libnvidia-opencl.so.%{version}
%{nvidia_libdir}/libnvidia-opencl.so.1
%{nvidia_libdir32}/libnvidia-opencl.so.*
%{nvidia_libdir32}/libnvcuvid.so.%{version}
%{nvidia_libdir32}/libnvcuvid.so.1
%{nvidia_libdir32}/libnvidia-encode.so.1
%{nvidia_libdir32}/libnvidia-encode.so.%{version}
%{nvidia_libdir32}/libOpenCL.so.1.0.0
%{nvidia_libdir32}/libOpenCL.so.1.0
%{nvidia_libdir32}/libOpenCL.so.1
%{nvidia_libdir32}/libnvidia-compiler.so.%{version}
%{nvidia_libdir32}/libcuda.so.%{version}
%{nvidia_libdir32}/libcuda.so.1

%changelog

* Wed Aug 22 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 396.54-1
- (25cd37e) Updated to v396.54
- (25cd37e) Changes w.r.t. v396.45:
- (25cd37e) * Fixed a resource leak that could lead to reduced performance after
- (25cd37e) starting and stopping several OpenGL and/or Vulkan applications.
- (25cd37e) * Added support for Quadro P3200 with Max-Q Design.
- (25cd37e) * Added support for VK_EXT_vertex_attribute_divisor.

* Fri Jul 20 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 396.45-1
- (0b8b4e1) Updated to v396.45
- (0b8b4e1) https://devtalk.nvidia.com/default/topic/1037689/b/t/post/5271904/#5271904
- (0b8b4e1) Release highlights since 396.24:
- (0b8b4e1) Added support for the following GPU:
- (0b8b4e1) TITAN V JHH Special Edition
- (0b8b4e1) Improved recovery of Vulkan direct-to-display applications after an
- (0b8b4e1) application hang or crash.
- (0b8b4e1) Fixed a bug that could cause multi-threaded EGL applications to crash
- (0b8b4e1) when exiting.
- (0b8b4e1) Improved compatibility with recent Linux kernels.
- (0b8b4e1) Fixed an intermittent hang of Vulkan applications running fullscreen
- (0b8b4e1) when flipping is allowed.
- (0b8b4e1) Removed informational messages that were printed by nvidia-modeset.ko
- (0b8b4e1) whenever a GPU device was allocated or freed.
- (0b8b4e1) Fixed a bug that could cause kernel panics when using Quadro SDI
- (0b8b4e1) Capture hardware.
- (0b8b4e1) Fixed a bug that caused kwin OpenGL compositing to crash when launching
- (0b8b4e1) certain OpenGL applications.
- (0b8b4e1) Fixed an intermittent crash when launching Vulkan applications.
- (0b8b4e1) Fixed an intermittent crash when launching applications through Wine.
- (0b8b4e1) Fixed a bug that caused the driver, in some low bandwidth DisplayPort
- (0b8b4e1) configurations, to not implicitly enable display dithering. This
- (0b8b4e1) resulted in visible banding.
- (0b8b4e1) Fixed intermittent hangs of fullscreen Vulkan applications when
- (0b8b4e1) focused away (e.g., by using the alt-tab key combination) on
- (0b8b4e1) non-composited desktops.
- (0b8b4e1) Increased the version numbers of the GLVND libGL, libGLESv1_CM,
- (0b8b4e1) libGLESv2, and libEGL libraries, to prevent concurrently installed
- (0b8b4e1) non-GLVND libraries from taking precedence in the dynamic linker cache.
- (0b8b4e1) Updated the OpenGL driver to allow the use of integer format (SINT/UINT)
- (0b8b4e1) color attachments with depth attachments in Frame Buffer Objects.

* Tue Jun 12 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 396.24-1
- (3837e82) Explicitly require EGL development files
- (Evgenii Shatokhin: 59a0616) Updated to v396.24
- (Evgenii Shatokhin: 59a0616) IMPORTANT:
- (Evgenii Shatokhin: 59a0616) * Staring from v396.x, 32-bit x86 systems are not supported by the driver.
- (Evgenii Shatokhin: 59a0616) 32-bit applications running in 64-bit systems (e.g. Steam) *are*
- (Evgenii Shatokhin: 59a0616) still supported. however.
- (Evgenii Shatokhin: 59a0616) * This version drops support for GeForce 400 and 500 series ("Fermi").
- (Evgenii Shatokhin: 59a0616) The users with such GPUs should use nvidia390 instead. As it was told
- (Evgenii Shatokhin: 59a0616) many times, nvidia-current is mostly for experiments and debugging as
- (Evgenii Shatokhin: 59a0616) well as for newer hardware which is not supported by nvidiaXYZ yet.
- (Evgenii Shatokhin: 59a0616) Other changes:
- (Evgenii Shatokhin: 59a0616) * v396.18 compared to v390.48: https://devtalk.nvidia.com/default/topic/1032146/
- (Evgenii Shatokhin: 59a0616) * v396.24 compared to v396.18: https://devtalk.nvidia.com/default/topic/1032986/

* Mon Jun 11 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 390.42-3
- (32809c3) Do not remake initrd when installing or removing the driver
- (32809c3) It was needed, because the drivers could get into the initrd and mess
- (32809c3) things up, even after the dkms package has been removed.
- (32809c3) dracut 046 (and some of the earlier our builds) generates generic, i.e.
- (32809c3) non-host-only, initrd images by default and should never place nvidia
- (32809c3) drivers there.
- (32809c3) Let us try without rebuilding initrd and save some time when installing
- (32809c3) and removing the driver.
- (Evgenii Shatokhin: b24e18c) Added a couple build options from Ubuntu: disable some build checks
- (Evgenii Shatokhin: 8f8e0bf) No more FTP downloads are available
- (Evgenii Shatokhin: 1a99136) Allow building the drivers with a different GCC version than the kernel
- (Evgenii Shatokhin: 1a99136) Might be risky but it is still better than failed builds of the drivers
- (Evgenii Shatokhin: 1a99136) resulting in broken graphics at the users' machines after a GCC update.
- (Evgenii Shatokhin: 1a99136) Besides, GCC can be updated in ROSA but these updates are usually rather
- (Evgenii Shatokhin: 1a99136) small within the scope of the given platform, e.g. 2016.1. They
- (Evgenii Shatokhin: 1a99136) should not break compatibility between the kernel and the drivers.
- (Evgenii Shatokhin: 1a99136) By the way, nvidia-installer seems to ignore the GCC version mismatches
- (Evgenii Shatokhin: 1a99136) right now. Let us do the same and allow different versions of GCC for the
- (Evgenii Shatokhin: 1a99136) kernel and the modules.
- (Evgenii Shatokhin: 1a99136) One could disable such check with an env. variable but let us just remove
- (Evgenii Shatokhin: 1a99136) it from the build scripts, for convenience.

* Sat Mar 17 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 390.42-2
- (39e1b92) Fixed the URLs
- (Evgenii Shatokhin: 1778549) Updated to v390.42 to be on par with nvidia390

* Sun Feb 18 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 390.25-1
- (114b767) Updated to version 390.25
- (114b767) https://devtalk.nvidia.com/default/topic/1029337/b/t/post/5235825/#5235825

* Tue Jan 16 2018 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 390.12-1
- (60b4974) Updated to version 390.12
- (60b4974) Release highlights:
- (60b4974) Added support for the following GPUs:
- (60b4974) * GeForce GTX 1050 Ti with Max-Q Design
- (60b4974) * GeForce GTX 1070 Ti
- (60b4974) * GeForce MX110
- (60b4974) * GeForce MX130
- (60b4974) * Quadro P500
- (60b4974) * TITAN V
- (60b4974) * TITAN Xp COLLECTORS EDITION
- (60b4974) * Corrected the SONAME of the copy of the libnvidia-egl-wayland library
- (60b4974) included in the .run installer package to libnvidia-egl-wayland.so.1.
- (60b4974) The SONAME had previously been versioned incorrectly with the full
- (60b4974) version number of the library.
- (60b4974) * Updated nvidia.ko to veto the ACPI_VIDEO_NOTIFY_PROBE event on kernels
- (60b4974) that allow the handler for this event to be overridden, to improve
- (60b4974) interaction between the NVIDIA driver and acpi_video on display hotplug
- (60b4974) events.
- (60b4974) * Updated the X driver's composition pipeline (used for rotation, warp
- (60b4974) and blend, transformation matrices, etc) to also support stereo.
- (60b4974) * Added an OpenGL stereo preview feature to the screen page in
- (60b4974) nvidia-settings.
- (60b4974) * Fixed a bug where GetTexSubImage() would read incorrect data into a pixel
- (60b4974) buffer object when supplied with a target of GL_TEXTURE_1D_ARRAY and a
- (60b4974) non-zero yoffset value.
- (60b4974) * Added support for generic active stereo with in-band DisplayPort
- (60b4974) signaling. The X configuration option "InbandStereoSignaling" is
- (60b4974) deprecated in favor of this stereo mode.
- (60b4974) * Modified the driver to avoid restoring framebuffer console modes
- (60b4974) on virtual reality head-mounted displays.
- (60b4974) * Fixed a bug that could cause a system crash when using the new
- (60b4974) NVreg_EnableBacklightHandler kernel module parameter on GPUs with no displays connected.
- (60b4974) More info:
- (60b4974) https://devtalk.nvidia.com/default/topic/1028278/unix-graphics-announcements-and-news/linux-solaris-and-freebsd-driver-390-12-beta-/

* Tue Oct 31 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 387.22-1
- (59b5253) Updated to version 387.22
- (59b5253) https://devtalk.nvidia.com/default/topic/1025794/b/t/post/5217161/#5217161
- (59b5253) This version:
- (59b5253) * adds support for GeForce GTX 1070 Ti and a few more GPUs;
- (59b5253) * implements experimental handling of laptop backlight brightness
- (59b5253) (disabled by default);
- (59b5253) * adds G-SYNC to all supported Vulkan swapchains for Maxwell and up;
- (59b5253) * provides several bug fixes compared to v387.12

* Sat Oct 07 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 387.12-1
- (57e0540) Updated to version 387.12 beta
- (57e0540) https://devtalk.nvidia.com/default/topic/1024719/unix-graphics-announcements-and-news/linux-solaris-and-freebsd-driver-387-12-beta-/

* Sat Oct 07 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 384.90-1
- (c28cc60) Create a correct nvidia_icd.json file from the provided template
- (Evgenii Shatokhin: c12a169) Updated to version 384.90 to avoid being older than the long-lived drivers
- (Evgenii Shatokhin: c12a169) https://devtalk.nvidia.com/default/topic/1024243/unix-graphics-announcements-and-news/linux-solaris-and-freebsd-driver-384-90-long-lived-branch-release-/

* Thu Sep 28 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 384.47-1
- (8bc7b3e) A couple cosmetic fixes
- (Evgenii Shatokhin: 184e42a) Updated to version 384.47 (beta)
- (Evgenii Shatokhin: 184e42a) https://devtalk.nvidia.com/default/topic/1016125/b/t/post/5176222/#5176222

* Sat May 13 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 381.22-2
- (50fc192) Require the development files for dbus for the build
- (Evgenii Shatokhin: 2129d7d) Updated to version 381.22
- (Evgenii Shatokhin: 2129d7d) Changes since 378.13:
- (Evgenii Shatokhin: 2129d7d) - Added support for the following GPUs:
- (Evgenii Shatokhin: 2129d7d) GeForce GTX 1080 Ti
- (Evgenii Shatokhin: 2129d7d) Quadro M520
- (Evgenii Shatokhin: 2129d7d) TITAN Xp
- (Evgenii Shatokhin: 2129d7d) - Restored support for the following GPU:
- (Evgenii Shatokhin: 2129d7d) GRID K520
- (Evgenii Shatokhin: 2129d7d) - Improved compatibility with recent kernels.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused "nvidia-settings --query all" to print many
- (Evgenii Shatokhin: 2129d7d) duplicate entries.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused applications to crash in some situations when
- (Evgenii Shatokhin: 2129d7d) calling glXMakeCurrent while OpenGL threaded optimizations were
- (Evgenii Shatokhin: 2129d7d) enabled.  This frequently occurred when Steam was attempting to make a
- (Evgenii Shatokhin: 2129d7d) video appear full-screen.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused VDPAU applications to use the blit
- (Evgenii Shatokhin: 2129d7d) presentation queue when a previous VDPAU application didn't shut down
- (Evgenii Shatokhin: 2129d7d) cleanly.
- (Evgenii Shatokhin: 2129d7d) - Fixed hangs and crashes that could occur when an OpenGL context is
- (Evgenii Shatokhin: 2129d7d) created while the system is out of available memory.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused corruption when OpenGL windows were moved or
- (Evgenii Shatokhin: 2129d7d) resized.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused X screens that use Option "UseDisplayDevice"
- (Evgenii Shatokhin: 2129d7d) "none" to be resized to 640x480 when using "xrandr -s" to change the
- (Evgenii Shatokhin: 2129d7d) screen configuration.
- (Evgenii Shatokhin: 2129d7d) - Fixed a kernel crash that occurred when attempting to map large user
- (Evgenii Shatokhin: 2129d7d) memory allocations into CUDA.
- (Evgenii Shatokhin: 2129d7d) - Disabled OpenGL threaded optimizations by default, initially enabled
- (Evgenii Shatokhin: 2129d7d) in 378.09, due to various reports of instability.
- (Evgenii Shatokhin: 2129d7d) - Added support for the following several modre Vulkan extensions (they
- (Evgenii Shatokhin: 2129d7d) require a Vulkan loader version >= 1.0.42).
- (Evgenii Shatokhin: 2129d7d) - Removed the X driver's logo splash screen and the corresponding NoLogo
- (Evgenii Shatokhin: 2129d7d) and LogoPath xorg.conf options.
- (Evgenii Shatokhin: 2129d7d) - Added the "ResamplingMethod" MetaMode option, adding support for
- (Evgenii Shatokhin: 2129d7d) bicubic resampling methods when scaling screen transformations are in
- (Evgenii Shatokhin: 2129d7d) use. See the README for more details.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that left HDMI and DisplayPort audio muted after a
- (Evgenii Shatokhin: 2129d7d) framebuffer console mode was restored. For some displays, this caused
- (Evgenii Shatokhin: 2129d7d) the display to remain blank.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused audio over DisplayPort to stop working when
- (Evgenii Shatokhin: 2129d7d) the monitor was unplugged and plugged back in or awoken from DPMS
- (Evgenii Shatokhin: 2129d7d) power-saving mode.
- (Evgenii Shatokhin: 2129d7d) - Fixed a regression that caused corruption in certain applications,
- (Evgenii Shatokhin: 2129d7d) such as window border shadows in Unity, after resuming from suspend.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused the GLX_EXT_buffer_age buffer age extension to
- (Evgenii Shatokhin: 2129d7d) return incorrect values for stereo drawables when using passive stereo
- (Evgenii Shatokhin: 2129d7d) modes 5 through 9.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug in Vulkan direct to display where DP1.2 monitors were not
- (Evgenii Shatokhin: 2129d7d) being enumerated.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that could cause EGL applications to crash when calling
- (Evgenii Shatokhin: 2129d7d) eglInitialize() multiple times on X11-backed displays.
- (Evgenii Shatokhin: 2129d7d) - Fixed a regression that could cause rendering corruption on a monitor
- (Evgenii Shatokhin: 2129d7d) connected via DisplayPort upon a modeset event (for example, changing
- (Evgenii Shatokhin: 2129d7d) resolutions or power cycling the monitor).
- (Evgenii Shatokhin: 2129d7d) - Updated the display configuration page in the nvidia-settings control
- (Evgenii Shatokhin: 2129d7d) panel to accurately reflect HDMI 3D refresh rates.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that could cause OpenGL applications to crash when VT
- (Evgenii Shatokhin: 2129d7d) switching between multiple X servers.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused the system to become unresponsive after
- (Evgenii Shatokhin: 2129d7d) resuming from power management suspend/hibernate. Additional symptoms
- (Evgenii Shatokhin: 2129d7d) of this bug included display flickering and "Xid 56" errors in the
- (Evgenii Shatokhin: 2129d7d) kernel log.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused backlight brightness to not be controllable on
- (Evgenii Shatokhin: 2129d7d) some notebooks with DisplayPort internal panels.
- (Evgenii Shatokhin: 2129d7d) - Installation of the nvidia-drm kernel module is now optional. The new
- (Evgenii Shatokhin: 2129d7d) '--no-drm' option can be used to prevent nvidia-installer from
- (Evgenii Shatokhin: 2129d7d) building and installing nvidia-drm, on systems where this kernel module
- (Evgenii Shatokhin: 2129d7d) fails to build and/or load.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that could cause some applications to crash when running
- (Evgenii Shatokhin: 2129d7d) with PRIME Sync.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused OpenGL apps to have excessive CPU usage when
- (Evgenii Shatokhin: 2129d7d) running with PRIME Sync but without native displays enabled.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that could cause PRIME Sync to deadlock in the kernel,
- (Evgenii Shatokhin: 2129d7d) particularly common on Linux 4.10.
- (Evgenii Shatokhin: 2129d7d) - Fixed a bug that caused PRIME Sync to run slowly on systems with
- (Evgenii Shatokhin: 2129d7d) Pascal GPUs.

* Sat May 13 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 378.13-2
- (d2ca7dc) Do not use FTP servers to download the driver archives
- (d2ca7dc) Details:
- (d2ca7dc) https://devtalk.nvidia.com/default/topic/1005867/unix-graphics-announcements-and-news/discontinuing-ftp-access-to-download-nvidia-com/
- (Evgenii Shatokhin: 70255df) Use external GLVND libraries
- (Evgenii Shatokhin: 70255df) http://bugs.rosalinux.ru/show_bug.cgi?id=7791

* Sun Mar 26 2017 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 378.13-1
- (4c1dec2) Silence rpmlint complaints about 'devel-file-in-non-devel-package'
- (Evgenii Shatokhin: e6492b1) Updated to version 378.13
- (Evgenii Shatokhin: e6492b1) Release highlights since version 375.26:
- (Evgenii Shatokhin: e6492b1) Added support for the following GPU(s):
- (Evgenii Shatokhin: e6492b1) Quadro GP100
- (Evgenii Shatokhin: e6492b1) Quadro P4000
- (Evgenii Shatokhin: e6492b1) Quadro P3000
- (Evgenii Shatokhin: e6492b1) Quadro P2000
- (Evgenii Shatokhin: e6492b1) Quadro P1000
- (Evgenii Shatokhin: e6492b1) Quadro P600
- (Evgenii Shatokhin: e6492b1) Quadro P400
- (Evgenii Shatokhin: e6492b1) Quadro M1200
- (Evgenii Shatokhin: e6492b1) Quadro M2200
- (Evgenii Shatokhin: e6492b1) Disabled OpenGL threaded optimizations by default under Xinerama.
- (Evgenii Shatokhin: e6492b1) Fixed a bug that prevented PRIME Sync from working on notebooks with
- (Evgenii Shatokhin: e6492b1) GeForce GTX 4xx and 5xx series GPUs.
- (Evgenii Shatokhin: e6492b1) Fixed a bug that caused system hangs when resuming from suspend with
- (Evgenii Shatokhin: e6492b1) some GPUs.
- (Evgenii Shatokhin: e6492b1) Fixed a regression that could cause corruption when hot-plugging
- (Evgenii Shatokhin: e6492b1) displays.
- (Evgenii Shatokhin: e6492b1) Fixed a regression that prevented systems with multiple DisplayPort
- (Evgenii Shatokhin: e6492b1) monitors from resuming correctly from suspend.
- (Evgenii Shatokhin: e6492b1) Added support for the ARB_parallel_shader_compile extension to allow
- (Evgenii Shatokhin: e6492b1) multi-threaded compilation of GLSL shaders.
- (Evgenii Shatokhin: e6492b1) Updated the X driver to ignore any Virtual Reality Head Mounted Displays
- (Evgenii Shatokhin: e6492b1) (HMDs). See the "AllowHMD" X configuration option in the README for
- (Evgenii Shatokhin: e6492b1) details.
- (Evgenii Shatokhin: e6492b1) The driver will now advertise GLX FBConfigs with no depth bits on depth
- (Evgenii Shatokhin: e6492b1) 30 X screens.
- (Evgenii Shatokhin: e6492b1) Added support in nvidia-settings to view configured PRIME displays. To
- (Evgenii Shatokhin: e6492b1) enable PRIME displays, see "Offloading Graphics Display with RandR 1.4"
- (Evgenii Shatokhin: e6492b1) in the README.
- (Evgenii Shatokhin: e6492b1) Added infrastructure which enables the NVIDIA EGL driver to load EGL
- (Evgenii Shatokhin: e6492b1) external platform libraries that add client-side support for new window
- (Evgenii Shatokhin: e6492b1) systems, beyond the existing libnvidia-egl-wayland.so.1. For more
- (Evgenii Shatokhin: e6492b1) details, see:
- (Evgenii Shatokhin: e6492b1) https://github.com/NVIDIA/eglexternalplatform
- (Evgenii Shatokhin: e6492b1) https://github.com/NVIDIA/egl-wayland
- (Evgenii Shatokhin: e6492b1) Added support for the following Vulkan extensions:
- (Evgenii Shatokhin: e6492b1) VK_KHR_display
- (Evgenii Shatokhin: e6492b1) VK_KHR_display_swapchain
- (Evgenii Shatokhin: e6492b1) Enabled OpenGL threaded optimizations by default in the driver. Refer to
- (Evgenii Shatokhin: e6492b1) the "Threaded Optimizations" section in the "Specifying OpenGL
- (Evgenii Shatokhin: e6492b1) Environment Variable Settings" chapter of the README for details. These
- (Evgenii Shatokhin: e6492b1) optimizations will self-disable when they are degrading performance. As
- (Evgenii Shatokhin: e6492b1) a result, performance should be unchanged for many applications, and
- (Evgenii Shatokhin: e6492b1) increased for those that benefit from threaded optimizations and were
- (Evgenii Shatokhin: e6492b1) not already forcing them enabled.
- (Evgenii Shatokhin: e6492b1) Fixed a bug that caused hangs when using the NVIDIA driver on 32-bit x86
- (Evgenii Shatokhin: e6492b1) systems running older Linux kernel versions (e.g. 2.6.18).

* Wed Feb 15 2017 Andrey Bondrov <andrey.bondrov@rosalab.ru> 375.26-3
- (9f1a010) MassBuild#1250: Increase release tag

* Sun Dec 25 2016 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 375.26-2
- (cc466fd) Always re-create initrd when installing/removing these drivers
- (cc466fd) NVidia drivers may get into the initrd image in some cases. If a driver
- (cc466fd) is updated but initrd is not, the graphics will fail to start at the next
- (cc466fd) boot with an error similar to the following in dmesg:
- (cc466fd) -------------
- (cc466fd) NVRM: API mismatch: the client has the version 375.26, but
- (cc466fd) NVRM: this kernel module has the version 375.20.  Please
- (cc466fd) NVRM: make sure that this kernel module and all NVIDIA driver
- (cc466fd) NVRM: components have the same version.
- (cc466fd) -------------
- (cc466fd) Let is re-create initrd to make sure the correct driver is used.

* Sat Dec 24 2016 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 375.26-1
- (6fd4209) Updated to version 375.26
- (6fd4209) https://devtalk.nvidia.com/default/topic/981831/b/t/post/5037664/#5037664
- (6fd4209) Release highlights since 375.20:
- (6fd4209) * Added support for the following GPU:
- (6fd4209) Quadro M3000 SE
- (6fd4209) * Fixed a regression that prevented the backlight on some notebook
- (6fd4209) computer displays from turning back on after being shut off.
- (6fd4209) * Fixed a regression that prevented the console from being restored
- (6fd4209) properly when using SLI.
- (6fd4209) * Fixed a regression that could cause the nvidia-settings control
- (6fd4209) panel to crash on startup with certain GPU configurations.
- (6fd4209) * Fixed a regression that prevented the GLX_NV_swap_group extension
- (6fd4209) from working when using SLI or Quadro Sync.
- (6fd4209) * Fixed a long-standing bug that caused the NVIDIA X driver to not
- (6fd4209) correctly detect AC/battery hotplug events on notebooks.
- (6fd4209) * Fixed an interaction problem between the Linux kernel's
- (6fd4209) CONFIG_HARDENED_USERCOPY feature (new in Linux kernel 4.8) and
- (6fd4209) nvidia-drm.ko with modeset=1 enabled. This problem caused
- (6fd4209) CONFIG_HARDENED_USERCOPY to erroneously halt the system, issuing reports
- (6fd4209) of the form:
- (6fd4209) usercopy: kernel memory overwrite attempt detected to
- (6fd4209) ffff8803e8ec7ce0 (<process stack>) (8 bytes)

* Wed Dec 07 2016 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 375.20-2
- (aede852) Require dkms-nvidia-current explicitly
- (aede852) This is necessary in (probably rare) cases when nvidia-current is based
- (aede852) on the same driver version as some nvidiaXYZ.

* Wed Dec 07 2016 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 375.20-1
- (846f9bd) Updated to version 375.20
- (846f9bd) Upstream changes (w.r.t. v370.28):
- (846f9bd) Additions / improvements:
- (846f9bd) * Added support for the following GPUs:
- (846f9bd) GeForce GTX 1050
- (846f9bd) GeForce GTX 1050 Ti
- (846f9bd) Quadro M620
- (846f9bd) Quadro M5000 SE
- (846f9bd) Quadro P6000
- (846f9bd) Quadro P5000
- (846f9bd) Tesla P100-PCIE-12GB
- (846f9bd) Tesla P100-PCIE-16GB
- (846f9bd) Tesla P100-SXM2-16GB
- (846f9bd) Tesla P4
- (846f9bd) Tesla P40
- (846f9bd) * Added new X configuration options:
- (846f9bd) ForceCompositionPipeline
- (846f9bd) ForceFullCompositionPipeline
- (846f9bd) which override the MetaMode tokens with the same names.
- (846f9bd) * Added support for X.Org xserver ABI 23 (xorg-server 1.19)
- (846f9bd) * Added support for the RandR TILE property added in RandR 1.5.
- (846f9bd) * Improved compatibility with recent Linux kernels.
- (846f9bd) * Added support for NVIDIA 3D Vision 2 Stereo on Linux. This IR emitter
- (846f9bd) can be used with stereo mode "10" set in the X configuration file.
- (846f9bd) * Added support for the screen_info.ext_lfb_base field, on kernels that
- (846f9bd) have it, in order to properly handle UEFI framebuffer consoles with
- (846f9bd) physical addresses above 4GB.
- (846f9bd) * Improved console restore behavior on systems that use the UEFI
- (846f9bd) Graphics Output Protocol, and most vesafb modes.
- (846f9bd) * Added EGL support to the GL Vendor Neutral Dispatch Library
- (846f9bd) (libGLVND).
- (846f9bd) * Raised the on-disk OpenGL shader cache size from 64MB to 128MB. Refer
- (846f9bd) to the "Specifying OpenGL Environment Variable Settings" section of
- (846f9bd) the driver README for details on how to select its location or disable
- (846f9bd) it.
- (846f9bd) Fixes:
- (846f9bd) * Fixed a bug that could cause displays in SLI Mosaic to blank when
- (846f9bd) applying a transformation matrix.
- (846f9bd) * Fixed a bug that prevented 	G-SYNC from working on notebook
- (846f9bd) displays.
- (846f9bd) * Fixed a bug that caused DisplayPort multistream devices to go blank
- (846f9bd) after the console is restored.
- (846f9bd) * Fixed a bug that caused a kernel crash when starting X for the second
- (846f9bd) time when more than one DisplayPort multistream device is connected to
- (846f9bd) a single DisplayPort connector via a branch device.
- (846f9bd) * Fixed a crash in nvidia-settings when adding Application Profile Rule
- (846f9bd) and Profile entries.
- (846f9bd) * Removed the "Enable Tooltip" option in nvidia-settings for the GTK 2
- (846f9bd) interface.
- (846f9bd) * Fixed a bug in synchronization primitives shared between X and OpenGL.
- (846f9bd) * Fixed a bug that caused issues with panning and cursor constraining
- (846f9bd) when mixing PRIME-driven displays with natively driven displays.
- (846f9bd) * Fixed a bug that caused long delays when leaving the VT or disabling a
- (846f9bd) display device while an OpenGL application is running.
- (846f9bd) * Fixed a bug that could prevent systems with GPUs configured in SLI
- (846f9bd) from resuming correctly from ACPI S3/S4 sleep/hibernate states.
- (846f9bd) * Fixed a bug that could cause quad-buffered stereo applications to
- (846f9bd) crash when disabling displays.
- (846f9bd) * Updated the behavior of the nvidia-settings control panel upon closing
- (846f9bd) the control panel window to be consistent with clicking the control
- (846f9bd) panel's "Quit" button.
- (846f9bd) Packaging-related changes:
- (846f9bd) The .manifest file has duplicate lines this time. Perhaps, that is not
- (846f9bd) intentional. We take care of that anyway.

* Fri Sep 09 2016 Evgenii Shatokhin <eugene.shatokhin@rosalab.ru> 370.28-1
- (7cda827) Updated to versio 370.28


